# SceneTweaker v0.5
# Copyright Adam Billyard 2010
#

module LMap


class SceneTweakTool
	@@frustumscale = 10
	
	def activate

		@mousex = 0
		@mousey = 0
		Sketchup.active_model.active_view.invalidate
	end

	def deactivate(view)
		view.invalidate
	end

	def suspend(view)
		view.invalidate
	end
	
	def resume(view)
		view.invalidate
	end

	def onMouseMove(flags, x, y, view)

		deltax = x - @mousex
		deltay = y - @mousey
		@mousex = x
		@mousey = y
		if @ppage
		
			if ((flags & CONSTRAIN_MODIFIER_MASK) == CONSTRAIN_MODIFIER_MASK) 
				lookat = @ppage.camera.target - @ppage.camera.eye
				lookat.normalize
				lookright = lookat.cross(Geom::Vector3d.new(0,0,1))
				lookright.normalize
				lookup = lookright.cross(lookat)

				lookright.length = deltax
				lookup.length = -deltay
				xform = Geom::Transformation.translation(lookright + lookup)
				eye = @ppage.camera.eye.transform!(xform)
				target = @ppage.camera.target.transform!(xform)
				@ppage.camera.set(eye, target, Geom::Vector3d.new(0,0,1))
				view.camera.set(eye, target, Geom::Vector3d.new(0,0,1)) if @oldcam
			elsif @oldcam
				pitch = view.camera.direction.dot(Geom::Vector3d.new(0,0,1))
				deltay = 0 if (pitch > 0.9) and (deltay < 0)
				deltay = 0 if (pitch < -0.9) and (deltay > 0)
				xform = Geom::Transformation.rotation(view.camera.eye, view.camera.xaxis, -deltay * 0.002)
				xform = xform * Geom::Transformation.rotation(view.camera.eye, Geom::Vector3d.new(0,0,1), -deltax * 0.002)
				target = view.camera.target
				target.transform!(xform)
				@ppage.camera.set(view.camera.eye, target, Geom::Vector3d.new(0,0,1))
				view.camera.set(view.camera.eye, target, Geom::Vector3d.new(0,0,1))
			end
		
			@viewedit = true
			view.invalidate
		end

	end

	def removehandles
		Sketchup.active_model.entities.erase_entities(@handles) if @handles
		@handles = []
	end

	def addhandles
		removehandles
		for page in Sketchup.active_model.pages
			@handles += [Sketchup.active_model.entities.add_cline(page.camera.eye, page.camera.target)]
		end
		@handles
	end


	def onLButtonDown(flags, x, y, view)

		@mousex = x
		@mousey = y

		Sketchup.active_model.start_operation("Scene Internal")
		addhandles

		ph = view.pick_helper
		view.lock_inference
		ph.do_pick(x,y)
		thispick = ph.best_picked
		
		@oldcam = nil
		@ppage = nil
		if thispick.kind_of?(Sketchup::ConstructionLine)
			for pindex in 0..@handles.length
				if (thispick == @handles[pindex])
					@ppage =  Sketchup.active_model.pages[pindex]
				end
			end
		end
		removehandles
		Sketchup.active_model.abort_operation

		if @ppage
			if (flags & ALT_MODIFIER_MASK) == ALT_MODIFIER_MASK
				Sketchup.active_model.start_operation("Scene Tweak")
				@oldcam = [view.camera.eye, view.camera.target, Geom::Vector3d.new(0,0,1), view.camera.perspective?, view.camera.fov]
				view.camera = Sketchup::Camera.new(@ppage.camera.eye, @ppage.camera.target, Geom::Vector3d.new(0,0,1), @ppage.camera.perspective?, @ppage.camera.fov)
				view.invalidate
				@viewedit = false
			end
		end

	end

	def onLButtonUp(flags, x, y, view)

		if @oldcam
			view.camera = Sketchup::Camera.new(@oldcam[0],@oldcam[1],@oldcam[2],@oldcam[3],@oldcam[4])
			view.invalidate
			if @viewedit
				Sketchup.active_model.commit_operation
			else
				Sketchup.active_model.abort_operation
			end
		end
		@ppage = nil
	end
	

	def draw(view)
	
		aspect = view.vpwidth.to_f / view.vpheight.to_f
		lookat = view.camera.target - view.camera.eye
		view.drawing_color = Sketchup::Color.new "green"
		for page in Sketchup.active_model.pages
			v = view.camera.eye - page.camera.eye
			view.draw_text(view.screen_coords(page.camera.eye), page.name) if v.dot(lookat) < 0
		end

		refup = Geom::Vector3d.new(0,0,1)
		gazes = []
		frustae = []
		for page in Sketchup.active_model.pages	
			fov = 1
			fov = page.camera.fov if page.camera.perspective?
			vww = Math.tan(fov * 0.5 * Math::PI / 180.0) * @@frustumscale
			lookat = page.camera.target - page.camera.eye
			gazes += [page.camera.eye, page.camera.target]
			lookat.length = @@frustumscale
			lookright = lookat.cross(refup)
			lookright.length = vww * aspect
			lookup = lookright.cross(lookat)
			lookup.length = vww			
			
			c1 = page.camera.eye + (lookat+lookright+lookup)
			c2 = page.camera.eye + (lookat-lookright+lookup)
			c3 = page.camera.eye + (lookat-lookright-lookup)
			c4 = page.camera.eye + (lookat+lookright-lookup)
			frustae += [page.camera.eye,c1, page.camera.eye,c2, page.camera.eye,c3, page.camera.eye,c4]
			frustae += [c1,c2, c2,c3, c3,c4, c4,c1]			
		end
		view.drawing_color = Sketchup::Color.new "blue"
		view.draw_lines gazes
		view.drawing_color = Sketchup::Color.new "cyan"
		view.draw_lines frustae

	end
end
end

if( not file_loaded?(__FILE__))

	tb = UI::Toolbar.new("SceneTweak")
	cmd = UI::Command.new("Tweaker") { 
		Sketchup.active_model.select_tool LMap::SceneTweakTool.new
	}
	cmd.small_icon = "scenetweak.png"
	cmd.large_icon = "scenetweak.png"
	cmd.tooltip = "Shift=dolly, Cmd=target"
	cmd.menu_text = ""
	tb.add_item(cmd)
	tb.show

    file_loaded(__FILE__)
end
